#!perl -w
#
# Tk Transaction Manager.
# Use all components
#
# makarow, demed
#
use strict;
use Tk();
use Tk::TM::Common;
use Tk::TM::Lang;
use Tk::TM::DataObject;
use Tk::TM::wgTable;
use Tk::TM::wgBlank;
use Tk::TM::wgActionBar;
use Tk::TM::wgMenu;

package Tk::TM::Lib;
1;
